package legacy;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 * CSV出力プログラムのメインクラスです。
 *
 */
public class Main {

    private static final Logger logger = Logger.getLogger(Main.class.getName());

    public static void main(String... args) throws Exception {
        if (logger.isLoggable(Level.CONFIG)) {
            logger.config("処理を開始します");
        }
        try {
            Parameter parameter = new Parameter(args);
            if (logger.isLoggable(Level.CONFIG)) {
                StringBuilder buf = new StringBuilder();
                buf.append("入力ファイル格納ディレクトリ：").append(parameter.srcDir);
                buf.append(System.lineSeparator());
                buf.append("出力CSVファイルパス：").append(parameter.output);
                buf.append(System.lineSeparator());
                buf.append("基準日：").append(parameter.basetime);
                logger.config(buf.toString());
            }
            Main main = new Main();
            main.run(parameter);
        } catch (Exception e) {
            logger.log(Level.SEVERE, "", e);
            throw e;
        } finally {
            if (logger.isLoggable(Level.CONFIG)) {
                logger.config("処理を終了します");
            }
        }
    }

    void run(Parameter parameter) throws IOException {

        List<Source> sources = extract(parameter);

        if (sources.isEmpty()) {
            if (logger.isLoggable(Level.CONFIG)) {
                logger.config("処理対象となるファイルはありませんでした");
            }
            return;
        }

        BufferedWriter out = new BufferedWriter(
                new OutputStreamWriter(new FileOutputStream(parameter.output), "UTF-8"));
        try {
            for (Source source : sources) {
                String csv = source.toCsv();
                out.write(csv);
                out.newLine();
            }
        } finally {
            out.close();
        }
    }

    /**
     * 処理対象となるJPEGファイルを集めて返します。
     * 
     * @param parameter
     * @return
     */
    List<Source> extract(Parameter parameter) {
        List<Source> sources = new ArrayList<Source>();
        Source.Builder builder = new Source.Builder(parameter);
        extract(builder, sources, parameter.srcDir);
        return sources;
    }

    void extract(Source.Builder builder, List<Source> sources, File file) {
        if (file.isDirectory()) {
            for (File fileOrDir : file.listFiles()) {
                extract(builder, sources, fileOrDir);
            }
        } else {
            Source source = builder.build(file);
            if (//*.jpg
                    source.suffix(".jpg")
                    //1バイト以上
                    && source.isNotEmpty()
                    //更新日時がbasetime以降
                    && source.isNewer()) {
                sources.add(source);
            }
        }
    }
}
