package legacy;

import java.io.File;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;

/**
 * CSV出力プログラムのパラメータをまとめたクラスです。
 *
 */
public class Parameter {
    /**
     * ファイルを絞り込む基準となる時刻。
     */
    public final long basetime;
    /**
     * 出力されるCSVのファイルパス。
     */
    public final File output;
    /**
     * 処理対象のJPEGファイルが格納されたディレクトリ。
     */
    public final File srcDir;

    public Parameter(String... args) {
        if (args.length != 2 && args.length != 3)
            throw new IllegalArgumentException("第1引数に入力ファイルが格納されたディレクトリのパス、"
                    + "第2引数に出力先となるCSVファイルのパス、"
                    + "第3引数(オプショナル)に基準日を渡さなければいけません");
        srcDir = new File(args[0]);
        if (!srcDir.isDirectory())
            throw new IllegalArgumentException("第1引数はディレクトリでなければいけません");

        output = new File(args[1]);
        if (!output.getName().endsWith(".csv"))
            throw new IllegalArgumentException("第2引数の拡張子は.csvでなければいけません");
        if (output.exists())
            throw new IllegalStateException("第2引数で指定するファイルパスは存在しないファイルでなければいけません");

        basetime = (args.length != 3) ? new Date().getTime() : parse(args[2]).getTime();
    }
    
    static Date parse(String text) {
        try {
            return new SimpleDateFormat("yyyyMMdd").parse(text);
        } catch (ParseException e) {
            throw new IllegalArgumentException("第3引数はyyyyMMdd形式の日付でなければいけません", e);
        }
    }
}
