package modern;

import java.io.BufferedWriter;
import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Path;
import java.util.List;
import java.util.StringJoiner;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * CSV出力プログラムのメインクラスです。
 *
 */
public class Main {

    private static final Logger logger = Logger.getLogger(Main.class.getName());

    public static void main(String... args) throws IOException {
        logger.config(() -> "処理を開始します");
        try {
            Parameter parameter = new Parameter(args);
            logger.config(() -> new StringJoiner(System.lineSeparator())
                    .add("入力ファイル格納ディレクトリ：" + parameter.srcDir)
                    .add("出力CSVファイルパス：" + parameter.output)
                    .add("基準日：" + parameter.basetime).toString());
            Main main = new Main();
            main.run(parameter);
        } catch (Exception e) {
            logger.log(Level.SEVERE, "", e);
            throw e;
        } finally {
            logger.config(() -> "処理を終了します");
        }
    }

    void run(Parameter parameter) throws IOException {

        List<Source> sources = extract(parameter);

        if (sources.isEmpty()) {
            logger.config(() -> "処理対象となるファイルはありませんでした");
            return;
        }

        try (BufferedWriter out = Files.newBufferedWriter(parameter.output)) {
            for (Source source : sources) {
                String csv = source.toCsv();
                out.write(csv);
                out.newLine();
            }
        }
    }

    /**
     * 処理対象となるJPEGファイルを集めて返します。
     * 
     * @param parameter
     * @return
     * @throws IOException
     */
    List<Source> extract(Parameter parameter) throws IOException {
        try (Stream<Path> stream = Files.walk(parameter.srcDir)) {
            Source.Builder builder = new Source.Builder(parameter);
            return stream.map(builder::build)
                    //*.jpg
                    .filter(Source.suffix(".jpg"))
                    //1バイト以上
                    .filter(Source::isNotEmpty)
                    //更新日時がbasetime以降
                    .filter(Source::isNewer)
                    //Listに変換する
                    .collect(Collectors.toList());
        }
    }
}
